import base64
from flask import Blueprint, request, jsonify, make_response, json
from app import db
from app.Site.models import Site
from app.Site.schemas import site_schema
from app.utils import token_required, create_thumbnail, base4_image, paginated_query, delete_record

site = Blueprint('sites', __name__)

@site.route('/api/v1/add/site', methods=['POST'])
@token_required(1, 2)
def create_site(current_user):
    try:
        site_dict = json.loads(request.form.get('formData'))
        img_content_type = ""
        image_data = ""
        if 'image' in request.files:
            image_file, img_content_type = create_thumbnail(request.files['image'])
            if image_file:
                image_data = base64.b64encode(image_file.read()).decode('utf-8')
        db.session.add(Site(**site_schema.load({
            'customer_id': request.args.get('customer_id'),
            'site_name': site_dict.get('site_name', '').strip(),
            'address': site_dict.get('address', '').strip(),
            'siteImg': image_data,
            'img_content_type': img_content_type,
            'latitude': site_dict.get('latitude', '').strip(),
            'longitude': site_dict.get('longitude', '').strip(),
            'status': site_dict.get('status', ''),
        })))
        db.session.commit()
        return make_response(jsonify({'message': 'Site added successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'failed added Site', "status": "error"}), 500)

@site.route('/site_beta', methods=['GET'])
@token_required(1, 2)
def get_site_beta(curent_user):
    try:
        customer_id = request.args.get('customer_id')
        if customer_id:
            site = Site.query.filter_by(customer_id=customer_id).first()  
            if site:
                return jsonify({"site": {
                    'id': site.site_id,
                    'name': site.site_name,
                }}), 200
            else:
                return jsonify({"error": "No Site Found for the given customer_id","site":""}), 200
        else:
            return jsonify({"error": "Please provide customer_id"}), 400
    except Exception as e:
        return make_response(jsonify({"Error": str(e)}), 500)
   
@site.route('/site', methods=['GET'])
@token_required(1, 2)
def get_site_detail(current_user):
    try:

        search = request.args.get('search')
        sites = Site.query.filter_by(customer_id=request.args.get('customer_id'))
        if search:
            sites = sites.filter(Site.site_name.ilike(f'%{search}%'))
        sites = sites.filter_by(status='active').all()
        if sites:
            serialized_sites = []
            for site in sites:
                if site.site_id is 1:
                    continue
                serialized_sites.append({
                    'id': site.site_id,
                    'name': site.site_name,
                    'customer_id': site.customer_id
                })
            return jsonify({"site": serialized_sites}), 200
        else:
            return jsonify({"error": "No Site Found for the given Customer ID!"}), 200
    except Exception as e:
        return make_response(jsonify({"Error": str(e)}), 500)

@site.route('/api/v1/get/site', methods=['GET'])
@token_required(1, 2)
def get_sites(current_user):
    try:
        sites_paginated, data = paginated_query(Site.query.order_by(Site.site_name.asc()),
                                                [Site.site_name], Site.customer_id)
        serialized_sites = []
        for site in sites_paginated:
            if site.site_id is 1:
                continue
            serialized_sites.append({
                'id': site.site_id,
                'customer_id': site.customer_id,
                'site_name': site.site_name,
                'address': site.address,
                'siteImg': base4_image(site.siteImg) if site.siteImg else None,
                'siteImg_content_type': site.img_content_type,
                'latitude': site.latitude,
                'longitude': site.longitude,
                'status': site.status,
            })
        if serialized_sites:
            return make_response(jsonify({'site': serialized_sites, "data": data}), 200)
        else:
            return make_response(jsonify({'message': 'No Site found', "data": data}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', "status": "error"}), 500)

@site.route('/api/v1/get/site_by_id/<int:site_id>', methods=['GET'])
@token_required(1, 2)
def get_site_by_id(current_user, site_id):
    try:
        site = Site.query.filter_by(site_id=site_id).first()
        if site:
            return make_response(jsonify({'site': {
                'id': site.site_id,
                'customer_id': site.customer_id,
                'site_name': site.site_name,
                'address': site.address,
                'latitude': site.latitude,
                'longitude': site.longitude,
                'status': site.status,
            }, "siteImg": {
                'logo': base4_image(site.siteImg) if site.siteImg else None,
                'logo_content_type': site.img_content_type,
            }}), 200)
        return make_response(jsonify({'message': 'site not found', "status": "error"}), 500)
    except Exception as e:
        return make_response(jsonify({'message': f'error: {e}', "status": "error"}), 500)

@site.route('/api/v1/update/site/<int:site_id>', methods=['PUT'])
@token_required(1, 2)
def update_site(current_user, site_id):
    try:
        site = Site.query.get(site_id)
        if not site:
            return make_response(jsonify({'message': 'Site not found', "status": "error"}), 404)
        site_dict = json.loads(request.form.get('formData'))
        loaded_site_data =site_schema.load({
            'site_name': site_dict.get('site_name').strip(),
            'address': site_dict.get('address').strip(),
            'latitude': site_dict.get('latitude').strip(),
            'longitude': site_dict.get('longitude').strip(),
            'status': site_dict.get('status')
        })
        if 'image' in request.files:
            image_file, img_content_type = create_thumbnail(request.files['image'])
            if image_file:
                image_data = base64.b64encode(image_file.read()).decode('utf-8')
                site.siteImg = image_data
                site.img_content_type = img_content_type
        for key, value in loaded_site_data.items():
            setattr(site, key, value)
        db.session.commit()
        return make_response(jsonify({'message': 'Site updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to update site', "status": "error"}), 500)

@site.route('/api/v1/delete/site/<int:site_id>', methods=['DELETE'])
@token_required(1, 2)
def delete_site(current_user, site_id):
    return delete_record(Site, current_user, site_id, "Site", "site_id")